package dev.pfaff.altargui.glfw;

import dev.pfaff.glass.glfw.GLFWwindow;
import dev.pfaff.log4truth.Logger;

public final class GLFWUIUtil {
	private final GLFWwindow window;
	private final DelayedInitGuiContext gui;
	private int framebufferWidth;
	private int framebufferHeight;
	private float contentScale;

	public GLFWUIUtil(GLFWwindow window, DelayedInitGuiContext gui) {
		this.window = window;
		this.gui = gui;
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		var fbSize = window.getFramebufferSize();
		var contentScale = window.getWindowContentScale();
		this.framebufferWidth = fbSize.width();
		this.framebufferHeight = fbSize.height();
		this.contentScale = contentScale.min();
		this.updateUI();
	}

	public void framebufferSizeChanged(int width, int height) {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		this.framebufferWidth = width;
		this.framebufferHeight = height;
		this.updateUI();
	}

	public void contentScaleChanged(float x, float y) {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		this.contentScale = Math.min(x, y);
		this.updateUI();
	}

	private void updateUI() {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		Logger.log(() -> STR."updateUI: framebufferWidth=\{framebufferWidth}, framebufferHeight=\{framebufferHeight}, contentScale=\{contentScale}");
		gui.onWindowResize(framebufferWidth, framebufferHeight, contentScale);
	}
}
