package dev.pfaff.altargui.glfw;

import dev.pfaff.altargui.host.Host;
import dev.pfaff.altargui.host.IWindow;
import dev.pfaff.altargui.util.listenable.ConstantValue;
import dev.pfaff.altargui.util.listenable.ValueListenable;
import dev.pfaff.dyl4j.BundledLibraryLinker;
import dev.pfaff.glass.Arch;
import dev.pfaff.glass.OS;
import dev.pfaff.glass.opengl.GL;
import org.jspecify.annotations.NonNull;
import org.jspecify.annotations.Nullable;

import java.io.IOException;
import java.io.InputStream;
import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;
import java.lang.foreign.SymbolLookup;

public final class HostImpl implements Host {
	private static final ConstantValue<Float> CONSTANT_VALUE_1F = new ConstantValue<>(1f);

	private static final IWindow WINDOW = new IWindow() {
		@Override
		public @NonNull ValueListenable<@NonNull Float> scaleFactorListenable() {
			return CONSTANT_VALUE_1F;
		}
	};

	@Override
	public @NonNull IWindow window() {
		return WINDOW;
	}

	@Override
	public @Nullable String getClipboardString() {
		return null;
//		try {
//			Transferable var2 = AWT_CLIPBOARD.getContents(null);
//			if (!var2.isDataFlavorSupported(DataFlavor.stringFlavor)) return null;
//			return (String)var2.getTransferData(DataFlavor.stringFlavor);
//		} catch (Throwable t) {
//			Logger.log(() -> "Caught error thrown while getting clipboard string", t, List.of(ERROR));
//			return null;
//		}
	}

	@Override
	public void setClipboardString(@NonNull MemorySegment string) {
//		try {
//			Transferable var2 = AWT_CLIPBOARD.setContents(new StringSelection(str));
//			if (!var2.isDataFlavorSupported(DataFlavor.stringFlavor)) return null;
//			return (String)var2.getTransferData(DataFlavor.stringFlavor);
//		} catch (Throwable t) {
//			Logger.log(() -> "Caught error thrown while getting clipboard string", t, List.of(ERROR));
//			return null;
//		}
	}

	@Override
	public @NonNull GL gl() {
		return GLGlobals.GL;
	}

	@Override
	public @NonNull SymbolLookup loadLibImage() {
		InputStream inputStream = HostImpl.class.getResourceAsStream(switch (OS.HOST) {
			case OS.Linux _ when Arch.HOST instanceof Arch.aarch64 -> "/natives/libimage_c-linux-arm64.so";
			case OS.macOS _ when Arch.HOST instanceof Arch.amd64 -> "/natives/libimage_c-macos-amd64.dylib";
			case OS.Windows _ when Arch.HOST instanceof Arch.amd64 -> "/natives/image_c-windows-amd64.dll";
			default -> throw dev.pfaff.glass.Host.makeUnsupportedOSArchException();
		});
		assert inputStream != null;
		try {
			return BundledLibraryLinker.createLookup(inputStream, Arena.ofAuto());
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
